package SimpleSGF;
use strict;
use warnings;
use Carp;
use LibertyCheck qw(process_lc);
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(process_sgf);

sub process_sgf {
  my ($in, $maxmoves, $realboard, $checkselfcapture) = @_;
  my $self = {};
  $self->{sgf} = readfile($in);
  $self->{sgf} =~ s/\R//g;
  $self->{sizes} = getsize($self->{sgf});
  ($self->{tsumego}, $self->{nomove}) = isittsumego($self->{sgf});
  organizesetup($self);
  removevars($self);
  removemoves($self, $maxmoves) if defined $maxmoves;
  $self->{aref} = convert($self);
  if ($realboard && !($self->{tsumego} || $self->{nomove})) {
    $self->{aref} = process_lc($self->{aref}, $self->{sizes}, $checkselfcapture);
  } else {
    $self->{aref} = removeduplicates($self);
  }

  return $self->{sizes}, $self->{tsumego}, $self->{aref}, $self->{nomove};
}

sub readfile {
  my $filename = shift;
  my $string = do {
    local $/;
    open my $fh, '<', $filename or croak "cannot open file $filename";
    <$fh>;
  };
  return $string;
}

sub removeduplicates {
  my $self = shift;
  my %hash = map { +"$_->[1]:$_->[2]" => $_->[0] } @{$self->{aref}}; # the + tells perl its a 'map block list' type of map
  my @a = map { [ $hash{$_}, split ':' ] } keys %hash;
  return \@a;
}

sub convert {
  my $self = shift;
  my @a;

  for (split ';', $self->{sgf}) {
    if (/(A)?([BW])\[([a-z])([a-z])\]/) {
      unless ($self->{tsumego} && !defined $1) {
        push @a, [$2, ord($3) - 97, ord($4) - 97];
      }
    }
  }

  return \@a
}

sub removemoves {
  my ($self, $max) = @_;
  my $count = 0;
  my $string;

  for (split ';', $self->{sgf}) {
    $count++ if /^B\[|^W\[/;
    $string .= $_.';';
    last if $count >= $max
  }

  $self->{sgf} = $string
}

sub removevars {
  my $self = shift;
  removecomments($self); # comments might contain a ')'
  my $result = $self->{sgf};
  if ($self->{sgf} =~ /(\(;.*?;.*?)\)/) { # bypass game info since it may contain a ')'
    $result = $1;
    $result =~ s/[()]//g;
  }

  $self->{sgf} = $result;
}

sub removecomments {
  my $self = shift;
  $self->{sgf} =~ s/C\[.*?(?<!\\)\]//g;
}

sub isittsumego { # decide it is if AW appears before B or W
  my $sgf = shift;
  my $awpos = index($sgf, 'AW[');
  my $tsumego = my $nomove = 0;
  if ($awpos > -1) { # if AW[ was found
    if ($sgf !~ /AW(\[pd\]\[dp\]|\[dp\]\[pd\]|\[dd\]\[pp\]|\[pp\]\[dd\])/) { # not a foolproof way to detect ancient setup games
      if ($sgf =~ /;(B|W)\[/) {
        $tsumego = 1 if $awpos < $-[0];
      } else {
        $nomove = 1;
      }
    }
  }
  return $tsumego, $nomove;
}

sub getsize {
  if ($_[0] =~ /SZ\[(\d+)(:(\d+))?\]/) {
    return defined $2 ? [$1 - 1, $3 - 1] : [$1 - 1, $1 - 1];
  } else {
    return [18, 18]
  }
}

sub organizesetup {
  my $self = shift;
  my $string;

  for my $node (split ';', $self->{sgf}) {
    my $temp;
    $temp .= findsetup($node, 'AB') if $node =~ /AB\[/;
    $temp .= findsetup($node, 'AW') if $node =~ /AW\[/;
    if ($temp) {
      $string .= "$temp;";
    } else {
      $string .= "$node;";
    }
  }

  $self->{sgf} = $string;
}

sub findsetup {
  my ($node, $tag) = @_;
  my $newsgf = '';

  for my $matches ($node =~ /$tag((?:\[..\])*)/g) {
    for ($matches =~ /\[(..)\]/g) {
      $newsgf .= join '', $tag, '[', $_, '];';
    }
  }

  return $newsgf
}

1;
